#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"dSFMT.h"

dsfmt_t dsfmt1;


double rand_gauss( double mu , double sigma ){	//rand : gauss
	double x1 = dsfmt_genrand_close_open(&dsfmt1);
	double x2 = dsfmt_genrand_close_open(&dsfmt1);
	double y  = sigma * ( sqrt(-2 * log(x1) ) * cos( 2*M_PI * x2)) + mu;
	return y;
}

double rand_cauchy(double x0 , double gamma){	//rand: cauchy
	double x = dsfmt_genrand_close_open(&dsfmt1);
	double y = x0 + gamma * tan( M_PI * (x - 0.5) );
	return y;
}

int double_comp(const void *_a, const void *_b) {	//for qsort
	double a = *(double *)_a;
	double b = *(double *)_b;
	
	if (a > b)		return -1;
	else if (a < b)	return 1;
	else			return 0;
}

double Variance(double *x , int num){
	double sum_x  = 0.0;
	double sum_xx = 0.0;
	int i;
	for(i=0;i<num;i++){
		sum_x  += x[i];
		sum_xx += x[i]*x[i];
	}
	return (sum_xx/num - pow(sum_x/num, 2));
}

double Covariance(double *x , double *y , int num){
	double sum_x  = 0.0;
	double sum_y  = 0.0;
	double sum_xy = 0.0;
	int i;
	for(i=0;i<num;i++){
		sum_x  += x[i];
		sum_y  += y[i];
		sum_xy += x[i] * y[i];
	}
	return ( sum_xy /num - (sum_x/num)*(sum_y/num) );
}

double Correlation( double *x , double *y , int num){
	double r;
	r = Covariance(x,y,num)/sqrt( Variance(x,num) * Variance(y,num));
	
	return r;
}

void rank(double *x , double *rank , int num){
	int i,j;
	double *tmp;
	
	tmp = (double *) calloc( num , sizeof(double) );
	for(i=0;i<num;i++)	tmp[i] = x[i];
	qsort( tmp , num , sizeof(double) , double_comp);
	for(i = 0 ; i< num; i++){
		for(j=0; j< num ; j++){
			if( x[i] == tmp[j]){
				rank[i] = num-j;
				break;
			}
		}
	}
}


int main( int argc , char *argv[] ){
	dsfmt_init_gen_rand(&dsfmt1,1);
	
	double a = atof(argv[1]);
	int i,j;
	double X,Y;
	double *x, *y;
	double *rank_x , *rank_y;
	double *tmp;
	int num = 2000;
	x = (double *)calloc( num , sizeof(double) );
	y = (double *)calloc( num , sizeof(double) );
	rank_x = (double *)calloc( num , sizeof(double) );
	rank_y = (double *)calloc( num , sizeof(double) );
	tmp = (double *)calloc( num , sizeof(double) );
	
	char * fn , *fn2;
	fn = (char *)calloc( 100 , sizeof(char) );
	fn2 = (char *)calloc( 100 , sizeof(char) );
	sprintf( fn , "xy_%s.dat",argv[1]);
	sprintf( fn2 , "correlation_%s.dat",argv[1]);
	
	FILE *fp , *fp2;
	if( (fp = fopen(fn, "w")) == NULL){
		printf("File Error 1\n");
		exit(1);
	}
	if( (fp2 = fopen(fn2, "w")) == NULL){
		printf("File Error 2 \n");
		exit(1);
	}
	
	
	
	for( i = 0 ; i< num ; i++){ //generate the number
		X = rand_gauss(0.0,10.0);
		Y = rand_gauss(0.0,10.0);
		x[i] = X;
		y[i] = a * X + Y;
		fprintf(fp , "%f	%f\n", x[i],y[i]);
	}
	
	//Pearson
	double r_pearson;
	r_pearson = Correlation(x,y,num);
	fprintf(fp2, "Peason : r=%f	,t=%f\n", r_pearson , fabs(r_pearson)*sqrt(num-2)/sqrt(1-r_pearson*r_pearson) );
	//
	//Spearman
	double r_spearman;
	rank( x, rank_x , num);
	rank( y, rank_y , num);
	r_spearman = Correlation(rank_x,rank_y,num);
	fprintf(fp2 , "Spearman : r=%f	t=%f\n", r_spearman , fabs(r_spearman)*sqrt(num-2)/sqrt(1-r_spearman*r_spearman) );
	//
	//Kendall
	double r_kendall;
	double P=0.0;
	for(i=0;i<num;i++){
		for(j=i+1; j<num ; j++){
			if( (rank_x[i] > rank_x[j] && rank_y[i] > rank_y[j]) || (rank_x[i] < rank_x[j] && rank_y[i] < rank_y[j]) ){
				P += 1.0;
			}
		}
	}
	r_kendall = 4*P/(num * (num -1)) -1 ;
	fprintf(fp2, "Kendall : r=%f	t=%f\n", r_kendall , fabs(r_kendall)/sqrt((double)(4*num+10)/(9*num*(num-1)))  );
	//
	
	
	
	fclose(fp);
	fclose(fp2);
	free(x);
	free(y);
	free(rank_x);
	free(rank_y);
	free(fn);
	free(fn2);
	free(tmp);
	return 0;
}



